unit Netsales;

interface

uses
  WinTypes, WinProcs, Messages, SysUtils, Classes, Controls, inifiles;

Type
  TNetSaleObject = class
  public
    ID: String[40];
    Info: TStringList;
    constructor create;
    destructor destroy; override;
    procedure SetInfoFromString(StringInfo: string);
    function GetInfoAsString: String;
  End;

 type


  TNetSales = class(TList)
    FCurrent: Integer;
    FOnChange: TNotifyEvent;
    procedure SetCurrent(const Value: integer);
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;

    function  IsCurrentValid: bool;
    procedure WriteToIni(Ini: TIniFile);
    procedure ReadFromIni(Ini: TIniFile);

    procedure Add_OneNoNotify(ID, Info: String);
    procedure Add_One(ID, Info: String);
    procedure Delete_Current;
      { get strings from Current object }
    function  GetCurrentID: string;
    function  GetCurrentInfo: TStringList;
    function  GetCurrentInfoAsString: String;
      { update strings in Current object }
    procedure UpdateCurrentID(aID: String);
    procedure UpdateCurrentInfoAsString(aInfo: String);
    procedure UpdateCurrentInfo(aInfo: TStringList);
      { navigation }
    procedure Next;
    procedure Prior;
    procedure Last;
    procedure First;
      { property }
    property  Current: Integer read FCurrent write SetCurrent;
    property  OnChange: TNotifyEvent read FOnChange write FOnChange;
  End;

var

  Net_Sales: TNetSales;


implementation

uses inf_cons, inf_util;


(***********************************************************

                      TNetSaleObject

**********************************************************)
constructor TNetSaleObject.Create;
Begin
  Inherited create;
  Info:= TStringList.Create;
End;

destructor TNetSaleObject.Destroy;
Begin
  Info.Destroy;
  Info:= nil;
  Inherited Destroy;
End;

procedure TNetSaleObject.SetInfoFromString(StringInfo: string);
Begin
  { wordwrap is ste to true on NetSalesInfo ( TMemo ) component }
  Info.Add(StringInfo);
End;

function TNetSaleObject.GetInfoAsString: String;
Begin
  result:= StringsToString(Info);
End;


(***********************************************************

                      TNetSales

**********************************************************)
constructor TNetSales.Create;
Begin
  inherited Create;
  FCurrent:= -1;
End;

procedure TNetSales.Clear;
var
  i: integer;
Begin
  if Count > 0 then
   for i:= Count - 1 downto 0 do
   Begin
     TNetSaleObject(Items[i]).Destroy;
     Items[i]:= nil;
   End;
   Inherited Clear;
   Current:= -1;
 End;


destructor TNetSales.Destroy;
Begin
  Clear;
  inherited Destroy;
End;


procedure TNetSales.SetCurrent(const Value: integer);
Begin
  if Value <> FCurrent then
  Begin
    FCurrent:= Value;
    if Assigned(FOnChange) then OnChange(Self);
  End;
End;

procedure TNetSales.WriteToIni(Ini: TIniFile);
var
  i: integer;
Begin
  if Count > 0 then
  Begin
    for i:= 0 to Count - 1 do
    Begin
      FCurrent:= i;
      Ini.WriteString(c_NetSalesSection + IntToStr(i+1),
           c_NetSalesID, GetCurrentID);
      Ini.WriteString(c_NetSalesSection + IntToStr(i+1),
           c_NetSalesInfo, GetCurrentInfoAsString);
    End;
    SetCurrent(0);
  End;
End;

procedure TNetSales.ReadFromIni(Ini: TIniFile);
var
  i: integer;
const
  ReadError = 'Nothing more';
Begin
  I:= 1;
  while CompareText(Ini.ReadString(c_NetSalesSection + IntToStr(i), c_NetSalesID,
    ReadError), ReadError) <> 0 do
  Begin
    Add_OneNoNotify(Ini.ReadString(c_NetSalesSection + IntToStr(i), c_NetSalesID, ReadError),
            Ini.ReadString(c_NetSalesSection + IntToStr(i), c_NetSalesInfo, ReadError));
    Inc(i);
  End;
   { Force to update }
  FCurrent:= -1;
  if Count > 0 then Current:= 0;
End;

procedure TNetSales.Add_One(ID, Info: String);
Begin
  Add_OneNoNotify(ID, Info);
  if Assigned(FOnChange) then OnChange(Self);
End;

procedure TNetSales.Add_OneNoNotify(ID, Info: String);
var
  NS: TNetSaleObject;
Begin
  NS:= TNetSaleObject.Create;
  NS.ID:= ID;
  NS.SetInfoFromString(Info);
   { where to delete depends on Current value }
  if ( Current = -1 ) or ( Current = Count - 1 ) then
  Begin
    Add(NS);
    FCurrent:= Count - 1; { LastItem }
  End else
  Begin
    Insert(Current, NS);
  End;
End;



procedure TNetSales.Delete_Current;
Begin
  if IsCurrentValid then
  Begin
    TNetSaleObject(Items[Current]).Destroy;
    Delete(Current);
    if Count >= Current then FCurrent:= Count - 1 ;
    if Count = 0 then FCurrent:= -1;
    if Assigned(FOnChange) then OnChange(Self);
  End;
End;


function TNetSales.IsCurrentValid: bool;
begin
  try
    if ( Count > 0 ) and ( Current < Count) and
     ( Current <> -1 ) and ( Items[Current] <> nil ) then
    result:= true else
      raise EInvalidOperation.Create('Invalid Item Reference in NetSales items list');
  except
    result:= false;
  End;
End;


procedure TNetSales.UpdateCurrentID(aID: String);
Begin
 if IsCurrentValid then
   TNetSaleObject(Items[Current]).ID:= aID;
End;

procedure TNetSales.UpdateCurrentInfoAsString(aInfo: String);
Begin
 if IsCurrentValid then
   TNetSaleObject(Items[Current]).SetInfoFromString(aInfo);
End;

procedure TNetSales.UpdateCurrentInfo(aInfo: TStringList);
Begin
 if IsCurrentValid and ( aInfo <> nil ) then
   TNetSaleObject(Items[Current]).Info.Assign(aInfo);
End;

function  TNetSales.GetCurrentID: string;
Begin
 if IsCurrentValid
  then result:= TNetSaleObject(Items[Current]).ID
  else result:= '';
End;

function  TNetSales.GetCurrentInfo: TStringList;
Begin
 if IsCurrentValid
  then result:= TNetSaleObject(Items[Current]).Info
  else result:= nil;
End;

function  TNetSales.GetCurrentInfoAsSTring: String;
Begin
 if IsCurrentValid
  then result:= TNetSaleObject(Items[Current]).GetInfoAsString
  else result:= '';
End;

procedure TNetSales.Next;
Begin
  SetCurrent(Current + 1);
End;

procedure TNetSales.Prior;
Begin
  SetCurrent(Current - 1);
End;

procedure TNetSales.Last;
Begin
  SetCurrent(Count - 1);
End;

procedure TNetSales.First;
Begin
  SetCurrent(0);
End;

end.
